"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const radash_1 = __importDefault(require("radash"));
const pulumi = __importStar(require("@pulumi/pulumi"));
const aws = __importStar(require("@pulumi/aws"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const pulumi_aws_lambda_api_1 = require("@exobase/pulumi-aws-lambda-api");
const builds_1 = require("@exobase/builds");
const path_1 = __importDefault(require("path"));
const cmdish_1 = __importDefault(require("cmdish"));
const main = ({ workingDir, }) => __awaiter(void 0, void 0, void 0, function* () {
    //
    //  READ PROJECT CONFIG
    //
    const context = (yield fs_extra_1.default.readJSON(path_1.default.join(workingDir, "context.json")));
    const { platform, service, deployment } = context;
    const config = deployment.config.stack;
    //
    //  SETUP PROVIDER
    //
    const provider = new aws.Provider("aws", {
        secretKey: platform.providers.aws.accessKeySecret,
        accessKey: platform.providers.aws.accessKeyId,
        region: platform.providers.aws.region,
    });
    //
    //  READ FUNCTIONS
    //
    const functions = (0, builds_1.getFunctionMap)({
        path: path_1.default.resolve(workingDir, 'source'),
        ext: 'ts'
    });
    //
    //  EXECUTE BUILD
    //
    yield onlyOnce('build', () => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, cmdish_1.default)('yarn && yarn build', { cwd: path_1.default.resolve(workingDir, 'source') });
        return true; // cached value
    }))();
    //
    //  CREATE API/LAMBDA RESOURCES
    //
    const envVarDict = deployment.config.environmentVariables.reduce((acc, ev) => (Object.assign(Object.assign({}, acc), { [ev.name]: ev.value })), {});
    const api = new pulumi_aws_lambda_api_1.AWSLambdaAPI(radash_1.default.dashCase(service.name), {
        sourceDir: path_1.default.join(workingDir, "source"),
        sourceExt: "ts",
        runtime: "nodejs14.x",
        functions,
        getZip: (func) => {
            return new pulumi.asset.FileArchive(path_1.default.resolve(workingDir, 'source', 'build', 'modules', func.module, `${func.function}.zip`));
        },
        getHandler: (func) => `${func.function}.default`,
        timeout: toNumber(config.timeout),
        memory: toNumber(config.memory),
        environmentVariables: Object.assign(Object.assign({}, envVarDict), { EXOBASE_PLATFORM: platform.name, EXOBASE_SERVICE: service.name }),
        domain: service.domain,
    }, { provider });
    return {
        url: service.domain ? service.domain.fqd : api.api.url,
    };
});
const toNumber = (value) => {
    if (radash_1.default.isString(value))
        return parseInt(value);
    return value;
};
const onlyOnce = (key, func) => () => __awaiter(void 0, void 0, void 0, function* () {
    const cacheFilePath = path_1.default.resolve(process.cwd(), '.exo-operation-cache');
    console.log('x--> ONLY ONCE:', cacheFilePath);
    const cache = yield (() => __awaiter(void 0, void 0, void 0, function* () {
        const [notExists, existing] = yield radash_1.default.try(() => fs_extra_1.default.readJSON(cacheFilePath))();
        console.log({ notExists, existing });
        if (!notExists)
            return existing;
        yield fs_extra_1.default.writeJSON(cacheFilePath, {});
        return {};
    }))();
    if (cache[key])
        return cache[key];
    const [err, result] = yield radash_1.default.try(func)();
    console.log({ err, result });
    yield fs_extra_1.default.writeJSON(cacheFilePath, Object.assign(Object.assign({}, cache), { [key]: result }));
    if (err)
        throw err;
    else
        return result;
});
exports.default = main;
//# sourceMappingURL=index.js.map